SystemOrganization addCategory: #'Sight-Concurrent'!
SystemOrganization addCategory: #'Sight-Concurrent-Tests'!
SystemOrganization addCategory: #'Sight-Concurrent-Events'!
SystemOrganization addCategory: #'Sight-Concurrent-Messages'!
SystemOrganization addCategory: #'Sight-Concurrent-Examples'!

Exception subclass: #ScWorkerDetected
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent'!

!ScWorkerDetected methodsFor: 'priv handling' stamp: 'mt 9/13/2011 10:31:36.093'!
defaultAction

	| process |
	process := Processor activeProcess.
	Project spawnNewProcessIfThisIsUI: process.
	process terminate.! !

!ScWorkerDetected methodsFor: 'priv handling' stamp: 'mt 9/13/2011 10:04:56.08'!
isResumable

	^ false! !

Actor subclass: #ScMaster
	instanceVariableNames: 'workers nextMessageId nextMessageIdMutex pendingMessages messageBroker messageBrokerSemaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent'!

!ScMaster class methodsFor: 'as yet unclassified' stamp: 'mt 9/13/2011 11:51:03.137'!
firstPort
	"Used by workers."
	
	^ 10000! !

!ScMaster methodsFor: 'workers' stamp: 'mt 9/12/2011 14:41:18.659'!
createWorker

	self createWorkers: 1.! !

!ScMaster methodsFor: 'workers' stamp: 'mt 9/23/2011 14:00:19.946'!
createWorkers: count

	| masterPID |
	masterPID := OSProcess thisOSProcess pid.
	
	Preferences setPreference: #warnIfNoChangesFile toValue: false.
	Smalltalk saveSession.
	
	masterPID = OSProcess thisOSProcess pid
		ifTrue: ["----- This is the master image -----"
			Preferences setPreference: #warnIfNoChangesFile toValue: true.
			count timesRepeat: [
				| newWorker |
				newWorker := ((ScWorker id: self nextWorkerId)
					createFakeScript;
					initializeOSProcess).
				self
					connect: newWorker
					signal: #processCompleted
					to: self workers
					selector: #remove:ifAbsent:
					pattern: #(0 =nil).
				self workers add: newWorker].
				^ self]
		ifFalse: [	"----- This is a worker image -----"
			self stop.
			ScWorkerActor installForId: (Smalltalk getSystemAttribute: 2) asInteger.
			ScWorkerActor instance deleteFakeScript.
			ScWorkerDetected signal].! !

!ScMaster methodsFor: 'system notifications' stamp: 'mt 9/15/2011 13:20:48.601'!
disableNotificationProcessing

	SystemChangeNotifier uniqueInstance
		noMoreNotificationsFor: self.! !

!ScMaster methodsFor: 'system notifications' stamp: 'mt 9/15/2011 13:20:42.869'!
enableNotificationProcessing

	self disableNotificationProcessing.
	
	SystemChangeNotifier uniqueInstance
		notify: self ofAllSystemChangesUsing: #processNotification:.! !

!ScMaster methodsFor: 'actions' stamp: 'mt 9/23/2011 14:30:55.275'!
evaluate: aMessage

	self messageReceived: aMessage.
	
	aMessage isResponse ifTrue: [
		(self workers detect: [:w | w id = aMessage workerId]) isBusy: false.
		messageBrokerSemaphore signal].! !

!ScMaster methodsFor: 'initialization' stamp: 'mt 9/23/2011 14:24:53.248'!
initialize

	super initialize.
	
	nextMessageIdMutex := Mutex new.
	messageBrokerSemaphore := Semaphore new.
	
	self startOn: self class firstPort.
	self enableNotificationProcessing.
	self initializeMessageBroker.! !

!ScMaster methodsFor: 'messages' stamp: 'mt 9/23/2011 14:33:09.024'!
initializeMessageBroker
	"Start a process to assign messages to workers that idle."
	
	messageBroker := [[
		| nextWorker |
		[(nextWorker := self workers
			detect: [:worker | worker isBusy not] ifNone: []) isNil] whileTrue: [
			messageBrokerSemaphore wait].
		
		nextWorker isBusy: true.
		nextWorker evaluate: self pendingMessages next] repeat] forkAt: Processor lowIOPriority.! !

!ScMaster methodsFor: 'accessing' stamp: 'mt 9/23/2011 13:56:36.112'!
messageBroker

	^ messageBroker! !

!ScMaster methodsFor: 'signals' stamp: 'mt 9/23/2011 13:31:51.563'!
messageReceived: aMessage

	self emit.! !

!ScMaster methodsFor: 'messages' stamp: 'mt 9/23/2011 13:53:30.025'!
nextMessageId

	nextMessageIdMutex critical: [
		^ nextMessageId
			ifNil: [nextMessageId := 1]
			ifNotNil: [nextMessageId := nextMessageId + 1]].! !

!ScMaster methodsFor: 'workers' stamp: 'mt 9/13/2011 11:34:10.317'!
nextWorkerId

	| id |
	id := 1.
	self workers do: [:worker |
		worker id = id
			ifTrue: [id := id + 1]
			ifFalse: [^ id]].
	^ id! !

!ScMaster methodsFor: 'accessing' stamp: 'mt 9/23/2011 13:34:25.985'!
pendingMessages

	^ pendingMessages ifNil: [pendingMessages := SharedQueue new]! !

!ScMaster methodsFor: 'system notifications' stamp: 'mt 9/23/2011 13:47:54.192'!
processNotification: anEvent
	
	ScSystemEventProcessor subclassesDo: [:processorClass | 
		| processor |
		processor := processorClass new.
		(processor supportedItemKinds includes: anEvent itemKind)
			ifTrue: [self send: (ScRequestSystemUpdate new event: (processor process: anEvent))]].! !

!ScMaster methodsFor: 'messages' stamp: 'mt 9/23/2011 14:52:02.884'!
send: aMessage
	"Assign an id to the message and enqueue it. Thread-safe."
	
	aMessage id: self nextMessageId.
	self pendingMessages nextPut: aMessage. "Copy?"
	messageBrokerSemaphore signal.! !

!ScMaster methodsFor: 'start/stop' stamp: 'mt 9/23/2011 14:00:01.526'!
stop

	self terminateWorkers.
	self disableNotificationProcessing.
	workers := nil.
	
	super stop.
	self messageBroker terminate.! !

!ScMaster methodsFor: 'workers' stamp: 'mt 9/15/2011 11:34:48.113'!
terminateWorkers

	self workers do: [:worker | worker quit].! !

!ScMaster methodsFor: 'accessing' stamp: 'mt 9/13/2011 11:33:44.119'!
workers
	
	^ workers ifNil: [workers := SortedCollection new sortBlock: [:w1 :w2 |
		w1 id  <= w2 id]]! !

Actor subclass: #ScWorkerActor
	instanceVariableNames: 'id master'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent'!

ScWorkerActor class
	instanceVariableNames: 'instance'!

ScWorkerActor class
	instanceVariableNames: 'instance'!

!ScWorkerActor class methodsFor: 'instance creation' stamp: 'mt 9/15/2011 12:56:03.466'!
installForId: anInteger

	instance ifNotNil: [^ instance].
	
	instance := self new id: anInteger.
	Utilities setAuthorInitials: ('worker_', (instance id printPaddedWith: $0 to: 2)).
	instance startOn: ScMaster firstPort + instance id.
	instance master: (ActorId localhostPort: ScMaster firstPort).
	
	^ instance! !

!ScWorkerActor class methodsFor: 'instance creation' stamp: 'mt 9/15/2011 09:37:51.936'!
instance

	^ instance! !

!ScWorkerActor class methodsFor: 'instance creation' stamp: 'mt 9/15/2011 09:39:18.132'!
uninstall

	instance ifNil: [^ self].
	instance stop.
	instance := nil.
	
	Utilities setAuthorInitials: ''.! !

!ScWorkerActor methodsFor: 'WORKAROUND' stamp: 'mt 9/15/2011 10:21:46.177'!
deleteFakeScript

	WorldState addDeferredUIMessage: [
		(FileDirectory on: Smalltalk imagePath) deleteFileNamed: self id asString].! !

!ScWorkerActor methodsFor: 'actions' stamp: 'mt 9/23/2011 14:14:39.213'!
evaluate: aMessage

	WorldState addDeferredUIMessage: [self master evaluate: aMessage evaluate].! !

!ScWorkerActor methodsFor: 'accessing' stamp: 'mt 9/15/2011 09:35:03.332'!
id

	^ id! !

!ScWorkerActor methodsFor: 'accessing' stamp: 'mt 9/15/2011 09:35:09.847'!
id: anInteger

	id := anInteger.! !

!ScWorkerActor methodsFor: 'accessing' stamp: 'mt 9/15/2011 12:48:29.843'!
master

	^ master! !

!ScWorkerActor methodsFor: 'accessing' stamp: 'mt 9/15/2011 12:48:40.244'!
master: anActorId

	master := anActorId.! !

!ScWorkerActor methodsFor: 'actions' stamp: 'mt 9/15/2011 09:42:10.915'!
quit

	self stop.
	Smalltalk quitPrimitive.! !

!ExternalWindowsOSProcess methodsFor: '*sight-concurrent' stamp: 'mt 9/12/2011 14:10:51.942'!
terminate

	self threads do: [:t | t terminate].! !

Object subclass: #ScFactorialExample
	instanceVariableNames: 'master results startTime current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Examples'!

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 14:49:32.24'!
addResult: aMessage

	aMessage isResponse ifTrue: [
		self results add: aMessage contents.
		(startTime to: Time now) asDuration asSeconds > 5
			ifTrue: [^ self].
			
		self requestNext].! !

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 15:08:03.657'!
computeMulti

	current := 1.
	startTime := Time now.
	results := nil.
	100 timesRepeat: [self requestNext].! !

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 14:45:24.91'!
computeSingle

	current := 1.
	startTime := Time now.
	
	[
		self results add: current factorial.
		current := current + 1.
		(startTime to: Time now) asDuration asSeconds > 5
			ifTrue: [^ self].
	] repeat.! !

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 14:36:18.864'!
master

	^ master! !

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 14:38:20.858'!
master: aMaster

	master := aMaster.
	self connect: master signal: #messageReceived: toSelector: #addResult:.! !

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 14:46:23.249'!
requestNext

	self master send: (ScRequestFactorial new integer: current).
	current := current + 1.! !

!ScFactorialExample methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 14:37:41.748'!
results

	^ results ifNil: [results := OrderedCollection new]! !

Object subclass: #ScMessage
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Messages'!

!ScMessage methodsFor: 'evaluation' stamp: 'mt 9/23/2011 11:51:34.427'!
evaluate

	self subclassResponsibility.! !

!ScMessage methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:12:17.924'!
id

	^ id! !

!ScMessage methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:12:25.587'!
id: anInteger

	id := anInteger.! !

!ScMessage methodsFor: 'testing' stamp: 'mt 9/23/2011 14:18:52.237'!
isRequest

	self subclassResponsibility.! !

!ScMessage methodsFor: 'testing' stamp: 'mt 9/23/2011 14:18:55.938'!
isResponse

	self subclassResponsibility.! !

ScMessage subclass: #ScRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Messages'!

!ScRequest methodsFor: 'testing' stamp: 'mt 9/23/2011 14:19:08.239'!
isRequest

	^ true! !

!ScRequest methodsFor: 'testing' stamp: 'mt 9/23/2011 14:19:02.851'!
isResponse

	^ false! !

!ScRequest methodsFor: 'testing' stamp: 'mt 9/23/2011 14:19:37.504'!
isResponseExpected

	^ true! !

ScRequest subclass: #ScRequestFactorial
	instanceVariableNames: 'integer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Examples'!

!ScRequestFactorial methodsFor: 'as yet unclassified' stamp: 'mt 9/23/2011 15:01:45.384'!
evaluate

	^ ScResponse new
		workerId: ScWorkerActor instance id;
		contents: self integer factorial! !

!ScRequestFactorial methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:46:57.664'!
integer

	^ integer! !

!ScRequestFactorial methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:47:03.508'!
integer: anInteger

	integer := anInteger.! !

ScRequest subclass: #ScRequestSystemUpdate
	instanceVariableNames: 'event'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Messages'!

!ScRequestSystemUpdate methodsFor: 'evaluation' stamp: 'mt 9/23/2011 13:46:25.864'!
evaluate

	ScSystemEventProcessor subclassesDo: [:processorClass |
		| processor |
		processor := processorClass new.
		(processor supportedItemKinds includes: self event itemKind)
			ifTrue: [processor execute: self event]].
	
	^ nil! !

!ScRequestSystemUpdate methodsFor: 'accessing' stamp: 'mt 9/23/2011 13:45:50.827'!
event

	^ event! !

!ScRequestSystemUpdate methodsFor: 'accessing' stamp: 'mt 9/23/2011 13:46:06.755'!
event: aSystemEvent

	event := aSystemEvent.! !

!ScRequestSystemUpdate methodsFor: 'testing' stamp: 'mt 9/23/2011 14:19:48.587'!
isResponseExpected

	^ false! !

ScRequest subclass: #ScRequestTestCaseResult
	instanceVariableNames: 'testCaseName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Messages'!

!ScRequestTestCaseResult methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:44:26.796'!
testCase

	^ Smalltalk at: self testCaseName! !

!ScRequestTestCaseResult methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:44:17.291'!
testCaseName

	^ testCaseName! !

!ScRequestTestCaseResult methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:44:11.047'!
testCaseName: aSymbol

	testCaseName := aSymbol.! !

ScMessage subclass: #ScResponse
	instanceVariableNames: 'contents workerId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Messages'!

!ScResponse methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:48:44.541'!
contents

	^ contents! !

!ScResponse methodsFor: 'accessing' stamp: 'mt 9/23/2011 11:48:53.844'!
contents: anObject

	contents := anObject.! !

!ScResponse methodsFor: 'evaluation' stamp: 'mt 9/23/2011 11:52:04.138'!
evaluate

	^ self contents! !

!ScResponse methodsFor: 'testing' stamp: 'mt 9/23/2011 14:19:18.166'!
isRequest

	^ false! !

!ScResponse methodsFor: 'testing' stamp: 'mt 9/23/2011 14:19:22.121'!
isResponse

	^ true! !

!ScResponse methodsFor: 'accessing' stamp: 'mt 9/23/2011 14:16:29.834'!
workerId
	"The worker that generated this response."
	
	^ workerId! !

!ScResponse methodsFor: 'accessing' stamp: 'mt 9/23/2011 14:16:41.064'!
workerId: anInteger
	
	workerId := anInteger.! !

Object subclass: #ScSystemEventProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Events'!

!ScSystemEventProcessor commentStamp: 'mt 10/4/2010 13:30:02.314' prior: 0!
My sublasses are used to handle incoming system change notifications in PSLocalClient and convert them into source code update objects.!

ScSystemEventProcessor subclass: #ScCategoryEventProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Events'!

!ScCategoryEventProcessor methodsFor: 'execution' stamp: 'mt 10/4/2010 13:49:20.926'!
execute: aSystemEvent

	aSystemEvent isAdded ifTrue: [SystemOrganization addCategory: aSystemEvent item].
	"aSystemEvent isModified ?"
	aSystemEvent isRemoved ifTrue: [SystemOrganization removeSystemCategory: aSystemEvent item].
	aSystemEvent isRenamed ifTrue: [SystemOrganization renameCategory: aSystemEvent oldName toBe: aSystemEvent newName].! !

!ScCategoryEventProcessor methodsFor: 'accessing' stamp: 'mt 10/4/2010 13:30:41.587'!
supportedItemKinds

	^ #(category)! !

ScSystemEventProcessor subclass: #ScClassEventProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Events'!

!ScClassEventProcessor methodsFor: 'execution' stamp: 'mt 10/4/2010 13:52:38.531'!
execute: aSystemEvent
	
	| eventItemClass |
	eventItemClass := (aSystemEvent instVarNamed: #environment) at: AbstractEvent classKind.
	
	aSystemEvent isAdded ifTrue: [Compiler evaluate: aSystemEvent item "class definition"].
	aSystemEvent isCommented ifTrue: [(Smalltalk at: eventItemClass) comment: aSystemEvent item "class comment"].
	aSystemEvent isModified ifTrue: [Compiler evaluate: aSystemEvent item "modified class definition"].
	aSystemEvent isRecategorized ifTrue: [(Smalltalk at: eventItemClass) category: aSystemEvent itemCategory].
	aSystemEvent isRemoved ifTrue: [(Smalltalk at: eventItemClass) removeFromSystem].
	aSystemEvent isRenamed ifTrue: [(Smalltalk at: aSystemEvent oldName) rename: aSystemEvent newName].
	aSystemEvent isReorganized ifTrue: [(Smalltalk at: eventItemClass) organization changeFromString: aSystemEvent item "class protocols"]! !

!ScClassEventProcessor methodsFor: 'processing' stamp: 'mt 10/4/2010 13:39:53.467'!
finishProcess: aSystemEvent on: aPreprocessedSystemEventCopy

	aPreprocessedSystemEventCopy
		instVarNamed: #item
		put: aSystemEvent item name asSymbol. "class name by default"
	
	(aSystemEvent isAdded or: [aSystemEvent isModified])
		ifTrue: [aPreprocessedSystemEventCopy
			instVarNamed: #item
			put: aSystemEvent item definition]. "class definition"
	
	aSystemEvent isModified
		ifTrue: [aPreprocessedSystemEventCopy oldItem: aSystemEvent oldItem name asSymbol]. "old class name"
	
	aSystemEvent isCommented
		ifTrue: [aPreprocessedSystemEventCopy
			instVarNamed: #item
			put: aSystemEvent item comment]. "class comment"
	
	aSystemEvent isReorganized
		ifTrue: [aPreprocessedSystemEventCopy
			instVarNamed: #item
			put: aSystemEvent item organization asString]. "class protocols"! !

!ScClassEventProcessor methodsFor: 'accessing' stamp: 'mt 10/4/2010 13:43:47.885'!
supportedItemKinds

	^ #(class)! !

ScSystemEventProcessor subclass: #ScMethodEventProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Events'!

!ScMethodEventProcessor methodsFor: 'execution' stamp: 'mt 10/4/2010 13:56:24.959'!
execute: aSystemEvent

	| eventItemClass |
	eventItemClass := (aSystemEvent instVarNamed: #environment) at: AbstractEvent classKind.
	
	(aSystemEvent isAdded or: [aSystemEvent isModified])
		ifTrue: [(ReadStream on: aSystemEvent item) fileIn "method fileOut source with timeStamps and protocols a.s.o."].
	
	aSystemEvent isRecategorized
		ifTrue: [(Smalltalk at: eventItemClass) organization classify: aSystemEvent item under: aSystemEvent itemProtocol].
		
	aSystemEvent isRemoved
		ifTrue: [(Smalltalk at: eventItemClass) removeSelector: aSystemEvent item].! !

!ScMethodEventProcessor methodsFor: 'support' stamp: 'mt 10/4/2010 14:02:38.264'!
fileOutSourceFor: aCompiledMethod
	"Better than just #getSource. Preserves Protocols."
	
	| strm |
	strm := WriteStream on: (String new: 1000).

	strm header; timeStamp.
	aCompiledMethod methodClass
		printMethodChunk: aCompiledMethod selector
		withPreamble: true
		on: strm
		moveSource: false
		toFile: 0.
		
	^ strm contents! !

!ScMethodEventProcessor methodsFor: 'processing' stamp: 'mt 10/4/2010 13:43:06.772'!
finishProcess: aSystemEvent on: aPreprocessedSystemEventCopy

	aPreprocessedSystemEventCopy
		instVarNamed: #item
		put: aSystemEvent item selector. "method selector by default"
		
	(aSystemEvent isAdded or: [aSystemEvent isModified])
		ifTrue: [aPreprocessedSystemEventCopy
			instVarNamed: #item
			put: (self fileOutSourceFor: aSystemEvent item)]. "method source" 
	
	aSystemEvent isModified
		ifTrue: [aPreprocessedSystemEventCopy oldItem: aSystemEvent oldItem selector]. "old selector"! !

!ScMethodEventProcessor methodsFor: 'accessing' stamp: 'mt 10/4/2010 13:44:02.217'!
supportedItemKinds

	^ #(method)! !

!ScSystemEventProcessor methodsFor: 'execution' stamp: 'mt 10/4/2010 13:49:41.509'!
execute: aSystemEvent
	"Replicate the given event in the system."! !

!ScSystemEventProcessor methodsFor: 'processing' stamp: 'mt 10/4/2010 13:38:20.794'!
finishProcess: aSystemEvent on: aPreprocessedSystemEventCopy
	"Do event specific conversions."
! !

!ScSystemEventProcessor methodsFor: 'processing' stamp: 'mt 9/15/2011 14:02:17.659'!
process: aSystemEvent
	"Generate a source code update."
	
	| result env |
	result := aSystemEvent copy.
	env := (aSystemEvent instVarNamed: #environment) copy.

	"Use custom environment."
	result instVarNamed: #environment put: env.

	"Generic transformations."
	result itemClass ifNotNil: [result itemClass: result itemClass name asSymbol].
	result itemRequestor ifNotNil: [result itemRequestor: nil]. "Cannot transport requestor instance anyway."
	result itemMethod ifNotNil: [result itemMethod: result itemMethod selector].
	
	self finishProcess: aSystemEvent on: result.
	
	^ result! !

!ScSystemEventProcessor methodsFor: 'accessing' stamp: 'mt 10/4/2010 13:03:31.248'!
supportedItemKinds

	^ #()! !

TestCase subclass: #ScMasterTests
	instanceVariableNames: 'sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Tests'!

!ScMasterTests methodsFor: 'running' stamp: 'mt 9/15/2011 11:44:38.651'!
defaultTimeout

	^ 10! !

!ScMasterTests methodsFor: 'running' stamp: 'mt 9/15/2011 12:56:03.317'!
setUp

	OSProcess thisOSProcess updateActiveChildren.

	super setUp.
	sut := ScMaster new.! !

!ScMasterTests methodsFor: 'running' stamp: 'mt 9/15/2011 11:56:42.024'!
tearDown

	sut terminateWorkers.
	[sut workers notEmpty] whileTrue: [self wait: 200].
	super tearDown.! !

!ScMasterTests methodsFor: 'tests' stamp: 'mt 9/15/2011 11:53:30.288'!
test01CreateAndTerminateWorker

	self
		assert: OSProcess thisOSProcess activeChildren isEmpty;
		assert: sut workers isEmpty.
		
	sut createWorker.
	
	self
		assert: 1 equals: OSProcess thisOSProcess activeChildren size;
		assert: 1 equals: sut workers size.

	sut workers first quit.
"	OSProcess thisOSProcess updateActiveChildren.
	
	self
		assert: OSProcess thisOSProcess activeChildren isEmpty;
		assert: sut workers isEmpty."! !

!ScMasterTests methodsFor: 'tests' stamp: 'mt 9/15/2011 12:40:26.997'!
test02NextWorkerId

	self assert: 1 equals: sut nextWorkerId.
	sut createWorker.
	self assert: 2 equals: sut nextWorkerId.
	sut createWorker.
	self assert: 3 equals: sut nextWorkerId.
	
	"sut workers first quit.
	[sut workers size = 2] whileTrue: [OSProcess thisOSProcess updateActiveChildren].

	self assert: 1 equals: sut nextWorkerId."! !

!ScMasterTests methodsFor: 'tests' stamp: 'mt 9/23/2011 11:14:03.016'!
test03NextMessageId

	self
		assert: 1 equals: sut nextMessageId;
		assert: 2 equals: sut nextMessageId;
		assert: 3 equals: sut nextMessageId.! !

!ScMasterTests methodsFor: 'tests' stamp: 'mt 9/13/2011 10:41:17.114'!
wait: milliseconds

	(Delay forMilliseconds: milliseconds) wait.
	OSProcess thisOSProcess updateActiveChildren.! !

TestCase subclass: #ScWorkerTests
	instanceVariableNames: 'sut actor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent-Tests'!

!ScWorkerTests methodsFor: 'running' stamp: 'mt 9/15/2011 10:45:29.607'!
setUp

	super setUp.
	sut := ScWorker id: 999.! !

!ScWorkerTests methodsFor: 'tests' stamp: 'mt 9/15/2011 10:57:08.953'!
test01Start

	| spy |
	spy := SignalSpy onSender: sut signal: #processCreated.
	actor := ScWorkerActor new.
	
	sut createFakeScript.
	
	self deny: sut isRunning.
	
	sut initializeOSProcess.! !

ActorId subclass: #ScWorker
	instanceVariableNames: 'id osProcess isBusy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Concurrent'!

!ScWorker class methodsFor: 'instance creation' stamp: 'mt 9/15/2011 12:56:03.404'!
id: anInteger

	^ (self localhostPort: ScMaster firstPort + anInteger)
		id: anInteger! !

!ScWorker methodsFor: 'WORKAROUND' stamp: 'mt 9/15/2011 10:18:08.229'!
createFakeScript
	"Create a fake script file to be used as argument."

	(StandardFileStream forceNewFileNamed: self id asString)
		nextPutAll: '  '; "Add two characters to let ZIP detection work. See String>>unzipped."
		close.! !

!ScWorker methodsFor: 'accessing' stamp: 'mt 9/13/2011 11:14:51.317'!
id

	^ id! !

!ScWorker methodsFor: 'accessing' stamp: 'mt 9/13/2011 11:14:57.677'!
id: anInteger

	id := anInteger.! !

!ScWorker methodsFor: 'initialization' stamp: 'mt 9/15/2011 10:50:27.598'!
initializeOSProcess

	osProcess := (ExternalOSProcess concreteClass
		programName: Smalltalk vmFileName 
		arguments: {'-headless'. Smalltalk imageName. self id asString} 
		initialEnvironment: nil).

	self processCreated.
		
	osProcess addDependent: self.
	osProcess value. "Run the process."! !

!ScWorker methodsFor: 'testing' stamp: 'mt 9/23/2011 14:23:00.437'!
isBusy

	^ isBusy ifNil: [false]! !

!ScWorker methodsFor: 'accessing' stamp: 'mt 9/23/2011 14:23:12.311'!
isBusy: aBoolean

	isBusy := aBoolean.! !

!ScWorker methodsFor: 'testing' stamp: 'mt 9/15/2011 09:49:41.046'!
isRunning

	^ self osProcess isRunning and: [self isConnected]! !

!ScWorker methodsFor: 'accessing' stamp: 'mt 9/13/2011 11:18:44.07'!
osProcess

	^ osProcess! !

!ScWorker methodsFor: 'signals' stamp: 'mt 9/15/2011 09:59:09.053'!
processCompleted

	self emit.! !

!ScWorker methodsFor: 'signals' stamp: 'mt 9/15/2011 09:53:55.415'!
processCreated

	self emit.! !

!ScWorker methodsFor: 'signals' stamp: 'mt 9/15/2011 09:58:33.551'!
processStarted

	self emit.! !

!ScWorker methodsFor: 'updating' stamp: 'mt 9/15/2011 10:01:47.237'!
update: aSymbol

	aSymbol == #runState ifTrue: [
		self osProcess isNotYetRunning ifTrue: [^ self processCreated].
		self osProcess isRunning ifTrue: [^ self processStarted].
		self osProcess isComplete ifTrue: [^ self processCompleted]].! !
